# Generated by Django 1.9.7 on 2016-07-05 17:44
from django.db import migrations

import djangocms_attributes_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0008_link_attributes'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, help_text='Optional. Link HTML tag attributes', verbose_name='link tag attributes'),
        ),
    ]



# Generated by Django 1.9.2 on 2016-02-26 14:19
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0005_auto_20151003_1710'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='cmsplugin_ptr',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='+', serialize=False, to='cms.CMSPlugin'),
        ),
    ]



from django.db import migrations, models

import djangocms_link.validators


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0011_fixed_null_values'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='external_link',
            field=models.URLField(default='', validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=None)], max_length=2040, blank=True, help_text='Provide a valid URL to an external website.', verbose_name='External link'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='link',
            name='mailto',
            field=models.EmailField(default='', max_length=255, verbose_name='Email address', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='link',
            name='phone',
            field=models.CharField(default='', max_length=255, verbose_name='Phone', blank=True),
            preserve_default=False,
        ),
    ]



from django.conf import settings


def is_select2_enabled():
    use_select2 = getattr(settings, 'DJANGOCMS_LINK_USE_SELECT2', False)
    is_installed = 'django_select2' in settings.INSTALLED_APPS
    return use_select2 and is_installed


if is_select2_enabled():
    from djangocms_link.fields_select2 import Select2PageSearchField as PageSearchField  # noqa
else:
    from cms.forms.fields import PageSelectFormField as PageSearchField  # noqa



from django.conf import settings
from django.test import TestCase

from djangocms_link.fields import PageSearchField, is_select2_enabled
from djangocms_link.fields_select2 import Select2PageSearchField


class LinkFieldTestCase(TestCase):

    def test_field_with_django_select2_extension(self):
        self.assertTrue('django_select2' in settings.INSTALLED_APPS)
        self.assertTrue(is_select2_enabled())
        settings.INSTALLED_APPS.remove("django_select2")
        self.assertFalse(is_select2_enabled())
        self.assertEqual(PageSearchField, Select2PageSearchField)



import warnings

from django.core.exceptions import ValidationError

from cms.api import add_plugin, create_page
from cms.models import Placeholder, StaticPlaceholder
from cms.test_utils.testcases import CMSTestCase

from djangocms_link.cms_plugins import LinkPlugin
from djangocms_link.models import AbstractLink

from .helpers import get_filer_file


class LinkPluginsTestCase(CMSTestCase):

    def setUp(self):
        self.file = get_filer_file()
        self.language = "en"
        self.home = create_page(
            title="home",
            template="page.html",
            language=self.language,
        )
        self.home.publish(self.language)
        self.page = create_page(
            title="content",
            template="page.html",
            language=self.language,
        )
        self.page.publish(self.language)
        self.static_page = create_page(
            title='static-content',
            template='static_placeholder.html',
            language='en',
        )
        self.placeholder = self.page.placeholders.get(slot="content")
        self.superuser = self.get_superuser()

    def tearDown(self):
        self.file.delete()
        self.page.delete()
        self.home.delete()
        self.static_page.delete()
        self.superuser.delete()

    def test_link_plugin(self):
        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=LinkPlugin.__name__,
            language=self.language,
            internal_link=self.page,
        )
        plugin.full_clean()
        self.assertEqual(plugin.plugin_type, "LinkPlugin")

    def test_plugin_structure(self):
        request_url = self.page.get_absolute_url(self.language) + "?toolbar_off=true"

        plugin = add_plugin(
            placeholder=self.placeholder,
            plugin_type=LinkPlugin.__name__,
            language=self.language,
            internal_link=self.page,
            name="Page link",
        )
        self.page.publish(self.language)
        self.assertEqual(plugin.get_plugin_class_instance().name, "Link")

        with self.login_user_context(self.superuser):
            response = self.client.get(request_url)

        self.assertContains(response, '<a href="/en/content/">Page link</a>')

    def test_full_plugin_render(self):
        request_url = self.get_add_plugin_uri(
            placeholder=self.placeholder,
            plugin_type=LinkPlugin.__name__,
            language=self.language,
        )
        data = {
            "template": "default",
            "external_link": "https://www.google.com",
            "name": "External link"
        }

        with self.login_user_context(self.superuser), warnings.catch_warnings():
            # hide the "DontUsePageAttributeWarning" warning when using
            # `get_add_plugin_uri` to get cleaner test results
            warnings.simplefilter("ignore")
            response = self.client.post(request_url, data)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, '<div class="success">')

    def test_optional_link(self):
        AbstractLink.link_is_optional = True

        plugin = add_plugin(
            self.page.placeholders.get(slot='content'),
            'LinkPlugin',
            'en',
        )
        self.assertIsNone(plugin.full_clean())

        AbstractLink.link_is_optional = False

        self.assertEqual(AbstractLink.link_is_optional, False)

        plugin = add_plugin(
            self.page.placeholders.get(slot='content'),
            'LinkPlugin',
            'en',
        )

        # should throw "Please provide a link." error
        with self.assertRaises(ValidationError):
            plugin.clean()

    def test_in_placeholders(self):
        plugin = add_plugin(
            self.page.placeholders.get(slot='content'),
            'LinkPlugin',
            'en',
            internal_link=self.page,
        )
        self.assertEqual(plugin.get_link(), '/en/content/')

        placeholder = Placeholder(slot="generated_placeholder")
        placeholder.save()

        plugin = add_plugin(
            placeholder,
            'LinkPlugin',
            'en',
            internal_link=self.static_page,
        )
        # the generated placeholder has no page attached to it, thus:
        self.assertEqual(plugin.get_link(), '//example.com/en/static-content/')

        static_placeholder = StaticPlaceholder.objects.create(
            name='content_static',
            code='content_static',
            site_id=1,
        )
        static_placeholder.save()

        plugin_a = add_plugin(
            static_placeholder.draft,
            'LinkPlugin',
            'en',
            internal_link=self.page,
        )

        plugin_b = add_plugin(
            static_placeholder.public,
            'LinkPlugin',
            'en',
            internal_link=self.static_page,
        )
        # static placeholders will always return the full path
        self.assertEqual(plugin_a.get_link(), '//example.com/en/content/')
        self.assertEqual(plugin_b.get_link(), '//example.com/en/static-content/')

    def test_file(self):
        plugin = add_plugin(
            self.page.placeholders.get(slot="content"),
            "LinkPlugin",
            "en",
            file_link=self.file,
        )
        self.assertIn("test_file.pdf", plugin.get_link())
        self.assertIn("/media/filer_public/", plugin.get_link())



# Generated by Django 2.2.2 on 2019-06-20 04:49

import django.db.models.deletion
from django.db import migrations

import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0011_auto_20190418_0137'),
        ('djangocms_link', '0013_fix_hostname'),
    ]

    operations = [
        migrations.AddField(
            model_name='link',
            name='file_link',
            field=filer.fields.file.FilerFileField(blank=True, help_text='If provided links a file from the filer app.', null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.File', verbose_name='File link'),
        ),
    ]



from django.forms.models import ModelForm
from django.utils.translation import gettext_lazy as _

from djangocms_attributes_field.widgets import AttributesWidget

from .fields import PageSearchField
from .models import Link


class LinkForm(ModelForm):
    internal_link = PageSearchField(
        label=_('Internal link'),
        required=False,
    )

    def for_site(self, site):
        # override the internal_link fields queryset to contains just pages for
        # current site
        # this will work for PageSelectFormField
        from cms.models import Page
        self.fields['internal_link'].queryset = Page.objects.drafts().on_site(site)
        # set the current site as a internal_link field instance attribute
        # this will be used by the field later to properly set up the queryset
        # this will work for PageSearchField
        self.fields['internal_link'].site = site
        self.fields['internal_link'].widget.site = site

    class Meta:
        model = Link
        exclude = ('page', 'position', 'placeholder', 'language', 'plugin_type')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['attributes'].widget = AttributesWidget()



from django import forms

from cms.models import Page

from django_select2.forms import ModelSelect2Widget


class Select2PageSearchFieldMixin:
    search_fields = [
        'title_set__title__icontains',
        'title_set__menu_title__icontains',
        'title_set__slug__icontains'
    ]

    def label_from_instance(self, obj):
        return obj.get_title()


class Select2PageSelectWidget(Select2PageSearchFieldMixin, ModelSelect2Widget):
    site = None

    # show entries when clicking on it
    def build_attrs(self, base_attrs, extra_attrs=None):
        default_attrs = {"data-minimum-input-length": 0}
        default_attrs.update(base_attrs)
        attrs = super().build_attrs(default_attrs, extra_attrs=extra_attrs)
        return attrs

    def get_queryset(self):
        if self.site:
            return Page.objects.drafts().on_site(self.site)
        return Page.objects.drafts()

    # we need to implement jQuery ourselves, see #180
    class Media:
        js = (
            "https://code.jquery.com/jquery-3.5.1.slim.min.js",
        )


class Select2PageSearchField(forms.ModelChoiceField):
    widget = Select2PageSelectWidget()

    def __init__(self, *args, **kwargs):
        kwargs['queryset'] = self.widget.get_queryset()
        super().__init__(*args, **kwargs)



from django.db import migrations, models

import djangocms_link.validators
from djangocms_link.models import HOSTNAME


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0012_removed_null'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='external_link',
            field=models.URLField(blank=True, help_text='Provide a valid URL to an external website.', max_length=2040, verbose_name='External link', validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=HOSTNAME)]),
        ),
    ]



# Generated by Django 2.2.2 on 2019-06-21 04:07
from django.db import migrations, models

import djangocms_link.validators


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0014_link_file_link'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='external_link',
            field=models.CharField(blank=True, help_text='Provide a link to an external source.', max_length=2040, validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=None)], verbose_name='External link'),
        ),
    ]



#!/usr/bin/env python
from tempfile import mkdtemp


HELPER_SETTINGS = {
    'INSTALLED_APPS': [
        'filer',
        'django_select2',
        'tests.utils',
    ],
    'CMS_LANGUAGES': {
        1: [{
            'code': 'en',
            'name': 'English',
        }]
    },
    'LANGUAGE_CODE': 'en',
    'THUMBNAIL_PROCESSORS': (
        'easy_thumbnails.processors.colorspace',
        'easy_thumbnails.processors.autocrop',
        'filer.thumbnail_processors.scale_and_crop_with_subject_location',
        'easy_thumbnails.processors.filters',
    ),
    'ALLOWED_HOSTS': ['localhost'],
    'DJANGOCMS_LINK_USE_SELECT2': True,
    'CMS_TEMPLATES': (
        ('page.html', 'Normal page'),
        ('static_placeholder.html', 'Page with static placeholder'),
    ),
    'FILE_UPLOAD_TEMP_DIR': mkdtemp(),
}


def run():
    from app_helper import runner
    runner.cms('djangocms_link')


if __name__ == '__main__':
    run()



import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '__first__'),
    ]

    operations = [
        migrations.CreateModel(
            name='Link',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, serialize=False, parent_link=True, auto_created=True, to='cms.CMSPlugin', primary_key=True)),
                ('name', models.CharField(verbose_name='name', max_length=256)),
                ('url', models.URLField(verbose_name='link', blank=True, null=True)),
                ('anchor', models.CharField(help_text='This applies only to page and text links.', blank=True, default='', max_length=128, verbose_name='anchor')),
                ('mailto', models.EmailField(help_text='An email address has priority over a text link.', blank=True, null=True, max_length=75, verbose_name='mailto')),
                ('phone', models.CharField(help_text='A phone number has priority over a mailto link.', blank=True, null=True, max_length=40, verbose_name='Phone')),
                ('target', models.CharField(verbose_name='target', blank=True, max_length=100, choices=[('', 'same window'), ('_blank', 'new window'), ('_parent', 'parent window'), ('_top', 'topmost frame')])),
                ('page_link', models.ForeignKey(help_text='A link to a page has priority over a text link.', on_delete=django.db.models.deletion.SET_NULL, blank=True, verbose_name='page', to='cms.Page', null=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



from django.db import migrations, models


def reset_null_values(apps, schema_editor):
    Link = apps.get_model('djangocms_link', 'Link')
    plugins = Link.objects.all()
    plugins.filter(external_link__isnull=True).update(external_link='')
    plugins.filter(mailto__isnull=True).update(mailto='')
    plugins.filter(phone__isnull=True).update(phone='')


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0010_adapted_fields'),
    ]

    operations = [
        migrations.RunPython(reset_null_values),
    ]



from django.test import TestCase

from djangocms_link.models import HOSTNAME
from djangocms_link.validators import IntranetURLValidator


class LinkValidatorTestCase(TestCase):

    def test_intranet_host_re(self):
        host = r'[a-z,0-9,-]{1,15}'
        host_re = (
            '(' + IntranetURLValidator.hostname_re
            + IntranetURLValidator.domain_re
            + IntranetURLValidator.tld_re +
            '|' + host + '|localhost)'
        )
        validator = IntranetURLValidator(
            intranet_host_re=host,
        )
        self.assertEqual(validator.host_re, host_re)
        self.assertIsNone(HOSTNAME)



from django.db import migrations, models

import djangocms_link.validators
from djangocms_link.models import HOSTNAME


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0003_auto_20150212_1310'),
    ]

    operations = [
        migrations.AlterField(
            model_name='link',
            name='anchor',
            field=models.CharField(help_text='This applies only to page and text links. Do <em>not</em> include a preceding "#" symbol.', max_length=128, verbose_name='anchor', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='link',
            name='url',
            field=models.CharField(blank=True, max_length=2048, null=True, verbose_name='link', validators=[djangocms_link.validators.IntranetURLValidator(intranet_host_re=HOSTNAME)]),
            preserve_default=True,
        ),
    ]



from django.contrib.sites.models import Site
from django.utils.translation import gettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from .forms import LinkForm
from .models import Link


class LinkPlugin(CMSPluginBase):
    model = Link
    form = LinkForm
    name = _('Link')
    text_enabled = True
    allow_children = True

    fieldsets = [
        (None, {
            'fields': (
                'name',
                ('external_link', 'internal_link'),
            )
        }),
        (_('Link settings'), {
            'classes': ('collapse',),
            'fields': (
                ('mailto', 'phone'),
                ('anchor', 'target'),
                ('file_link'),
            ),
        }),
        (_('Advanced settings'), {
            'classes': ('collapse',),
            'fields': (
                'template',
                'attributes',
            )
        }),
    ]

    @classmethod
    def get_render_queryset(cls):
        queryset = super().get_render_queryset()
        return queryset.select_related('internal_link')

    def get_render_template(self, context, instance, placeholder):
        return f'djangocms_link/{instance.template}/link.html'

    def render(self, context, instance, placeholder):
        context['link'] = instance.get_link()
        return super().render(context, instance, placeholder)

    def get_form(self, request, obj=None, **kwargs):
        form_class = super().get_form(request, obj, **kwargs)

        if obj and obj.page and hasattr(obj.page, 'site') and obj.page.site:
            site = obj.page.site
        elif self.page and hasattr(self.page, 'site') and self.page.site:
            site = self.page.site
        else:
            site = Site.objects.get_current()

        class Form(form_class):
            def __init__(self, *args, **kwargs):
                super().__init__(*args, **kwargs)
                self.for_site(site)

        return Form


plugin_pool.register_plugin(LinkPlugin)



"""
Enables the user to add a "Link" plugin that displays a link
using the HTML <a> tag.
"""
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import force_str
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from cms.models import CMSPlugin, Page

from djangocms_attributes_field.fields import AttributesField
from filer.fields.file import FilerFileField

from .validators import IntranetURLValidator


# Add additional choices through the ``settings.py``.
def get_templates():
    choices = [
        ('default', _('Default')),
    ]
    choices += getattr(
        settings,
        'DJANGOCMS_LINK_TEMPLATES',
        [],
    )
    return choices


HOSTNAME = getattr(
    settings,
    'DJANGOCMS_LINK_INTRANET_HOSTNAME_PATTERN',
    None
)

TARGET_CHOICES = (
    ('_blank', _('Open in new window')),
    ('_self', _('Open in same window')),
    ('_parent', _('Delegate to parent')),
    ('_top', _('Delegate to top')),
)


class AbstractLink(CMSPlugin):
    # used by django CMS search
    search_fields = ('name', )

    # allows link requirement to be changed when another
    # CMS plugin inherits from AbstractLink
    link_is_optional = False

    url_validators = [
        IntranetURLValidator(intranet_host_re=HOSTNAME),
    ]

    template = models.CharField(
        verbose_name=_('Template'),
        choices=get_templates(),
        default=get_templates()[0][0],
        max_length=255,
    )
    name = models.CharField(
        verbose_name=_('Display name'),
        blank=True,
        max_length=255,
    )
    # re: max_length, see: http://stackoverflow.com/questions/417142/
    external_link = models.CharField(
        verbose_name=_('External link'),
        blank=True,
        max_length=2040,
        validators=url_validators,
        help_text=_('Provide a link to an external source.'),
    )
    internal_link = models.ForeignKey(
        Page,
        verbose_name=_('Internal link'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        help_text=_('If provided, overrides the external link.'),
    )
    file_link = FilerFileField(
        verbose_name=_('File link'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
        help_text=_('If provided links a file from the filer app.'),
    )
    # other link types
    anchor = models.CharField(
        verbose_name=_('Anchor'),
        blank=True,
        max_length=255,
        help_text=_('Appends the value only after the internal or external link. '
                    'Do <em>not</em> include a preceding "#" symbol.'),
    )
    mailto = models.EmailField(
        verbose_name=_('Email address'),
        blank=True,
        max_length=255,
    )
    phone = models.CharField(
        verbose_name=_('Phone'),
        blank=True,
        max_length=255,
    )
    # advanced options
    target = models.CharField(
        verbose_name=_('Target'),
        choices=TARGET_CHOICES,
        blank=True,
        max_length=255,
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
        excluded_keys=['href', 'target'],
    )

    # Add an app namespace to related_name to avoid field name clashes
    # with any other plugins that have a field with the same name as the
    # lowercase of the class name of this model.
    # https://github.com/divio/django-cms/issues/5030
    cmsplugin_ptr = models.OneToOneField(
        CMSPlugin,
        related_name='%(app_label)s_%(class)s',
        parent_link=True,
        on_delete=models.CASCADE,
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name or str(self.pk)

    def get_short_description(self):
        if self.name and self.get_link():
            return f'{self.name} ({self.get_link()})'
        return self.name or self.get_link() or gettext('<link is missing>')

    def get_link(self):
        if self.internal_link:
            ref_page = self.internal_link
            link = ref_page.get_absolute_url()

            # simulate the call to the unauthorized CMSPlugin.page property
            cms_page = self.placeholder.page if self.placeholder_id else None

            # first, we check if the placeholder the plugin is attached to
            # has a page. Thus the check "is not None":
            if cms_page is not None:
                if getattr(cms_page, 'node', None):
                    cms_page_site_id = getattr(cms_page.node, 'site_id', None)
                else:
                    cms_page_site_id = getattr(cms_page, 'site_id', None)
            # a plugin might not be attached to a page and thus has no site
            # associated with it. This also applies to plugins inside
            # static placeholders
            else:
                cms_page_site_id = None

            # now we do the same for the reference page the plugin links to
            # in order to compare them later
            if cms_page is not None:
                if getattr(cms_page, 'node', None):
                    ref_page_site_id = ref_page.node.site_id
                else:
                    ref_page_site_id = ref_page.site_id
            # if no external reference is found the plugin links to the
            # current page
            else:
                ref_page_site_id = Site.objects.get_current().pk

            if ref_page_site_id != cms_page_site_id:
                ref_site = Site.objects._get_site_by_id(ref_page_site_id).domain
                link = f'//{ref_site}{link}'

        elif self.file_link:
            link = self.file_link.url

        elif self.external_link:
            link = self.external_link

        elif self.phone:
            link = 'tel:{}'.format(self.phone.replace(' ', ''))

        elif self.mailto:
            link = f'mailto:{self.mailto}'

        else:
            link = ''

        if (not self.phone and not self.mailto) and self.anchor:
            link += f'#{self.anchor}'

        return link

    def clean(self):
        super().clean()
        field_names = (
            'external_link',
            'internal_link',
            'mailto',
            'phone',
            'file_link',
        )
        anchor_field_name = 'anchor'
        field_names_allowed_with_anchor = (
            'external_link',
            'internal_link',
        )

        anchor_field_verbose_name = force_str(
           self._meta.get_field(anchor_field_name).verbose_name)
        anchor_field_value = getattr(self, anchor_field_name)

        link_fields = {
            key: getattr(self, key)
            for key in field_names
        }
        link_field_verbose_names = {
            key: force_str(self._meta.get_field(key).verbose_name)
            for key in link_fields.keys()
        }
        provided_link_fields = {
            key: value
            for key, value in link_fields.items()
            if value
        }

        if len(provided_link_fields) > 1:
            # Too many fields have a value.
            verbose_names = sorted(link_field_verbose_names.values())
            error_msg = _('Only one of {0} or {1} may be given.').format(
                ', '.join(verbose_names[:-1]),
                verbose_names[-1],
            )
            errors = {}.fromkeys(provided_link_fields.keys(), error_msg)
            raise ValidationError(errors)

        if (len(provided_link_fields) == 0
                and not self.anchor
                and not self.link_is_optional):

            raise ValidationError(
                _('Please provide a link.')
            )

        if anchor_field_value:
            for field_name in provided_link_fields.keys():
                if field_name not in field_names_allowed_with_anchor:
                    error_msg = _('%(anchor_field_verbose_name)s is not allowed together with %(field_name)s') % {
                        'anchor_field_verbose_name': anchor_field_verbose_name,
                        'field_name': link_field_verbose_names.get(field_name)
                    }
                    raise ValidationError({
                        anchor_field_name: error_msg,
                        field_name: error_msg,
                    })


class Link(AbstractLink):

    class Meta:
        abstract = False



import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields

from djangocms_link.models import get_templates


class Migration(migrations.Migration):

    dependencies = [
        ('djangocms_link', '0009_auto_20160705_1344'),
    ]

    operations = [
        migrations.AddField(
            model_name='link',
            name='template',
            field=models.CharField(default=get_templates()[0][0], max_length=255, verbose_name='Template', choices=get_templates()),
        ),
        migrations.RenameField(
            model_name='link',
            old_name='url',
            new_name='external_link',
        ),
        migrations.RenameField(
            model_name='link',
            old_name='page_link',
            new_name='internal_link',
        ),
        migrations.AlterField(
            model_name='link',
            name='anchor',
            field=models.CharField(help_text='Appends the value only after the internal or external link. Do <em>not</em> include a preceding "#" symbol.', max_length=255, verbose_name='Anchor', blank=True),
        ),
        migrations.AlterField(
            model_name='link',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(default=dict, verbose_name='Attributes', blank=True),
        ),
        migrations.AlterField(
            model_name='link',
            name='internal_link',
            field=models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, blank=True, to='cms.Page', help_text='If provided, overrides the external link.', null=True, verbose_name='Internal link'),
        ),
        migrations.AlterField(
            model_name='link',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Display name', blank=True),
        ),
        migrations.AlterField(
            model_name='link',
            name='target',
            field=models.CharField(blank=True, max_length=255, verbose_name='Target', choices=[('_blank', 'Open in new window'), ('_self', 'Open in same window'), ('_parent', 'Delegate to parent'), ('_top', 'Delegate to top')]),
        ),
    ]



import re

from django.core.validators import URLValidator


class IntranetURLValidator(URLValidator):
    """
    This is essentially the normal, Django URL Validator, but allows for
    "internal" machine-name only "hostnames" as defined by the RegEx pattern
    defined in settings as well as normal, FQD-based hostnames.

    Some examples:
    RFC1123 Pattern
        DJANGOCMS_LINK_INTRANET_HOSTNAME_PATTERN = r'[a-z,0-9,-]{1,15}'
    """

    ul = '\u00a1-\uffff'  # unicode letters range (must be a unicode string, not a raw string)

    # IP patterns
    ipv4_re = r'(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}'
    ipv6_re = r'\[[0-9a-f:\.]+\]'  # (simple regex, validated later)

    # Host patterns
    hostname_re = r'[a-z' + ul + r'0-9](?:[a-z' + ul + r'0-9-]*[a-z' + ul + r'0-9])?'
    domain_re = r'(?:\.[a-z' + ul + r'0-9]+(?:[a-z' + ul + r'0-9-]*[a-z' + ul + r'0-9]+)*)*'
    tld_re = r'\.[a-z' + ul + r']{2,}\.?'
    host_re = '(' + hostname_re + domain_re + tld_re + '|localhost)'

    def __init__(self, intranet_host_re=None, **kwargs):
        super().__init__(**kwargs)
        if intranet_host_re:
            self.host_re = (
                '(' + self.hostname_re + self.domain_re + self.tld_re +
                '|' + intranet_host_re + '|localhost)'
            )
            self.regex = re.compile(
                r'^(?:[a-z0-9\.\-]*)://'
                r'(?:\S+(?::\S*)?@)?'
                r'(?:' + self.ipv4_re + '|' + self.ipv6_re + '|' + self.host_re + ')'
                r'(?::\d{2,5})?'
                r'(?:[/?#][^\s]*)?'
                r'$', re.IGNORECASE)



from django.core.exceptions import ValidationError
from django.test import TestCase

from cms.api import create_page

from djangocms_link.models import TARGET_CHOICES, Link

from .helpers import get_filer_file


class LinkModelTestCase(TestCase):

    def setUp(self):
        self.page = create_page(
            title='help',
            template='page.html',
            language='en',
        )
        self.file = get_filer_file()
        self.link = Link.objects.create(
            template="default",
            name="My Link",
            internal_link=self.page,
            external_link="http://www.divio.com",
            file_link=self.file,
            anchor="some_id",
            mailto="test@email.com",
            phone="+01 234 567 89",
            target=TARGET_CHOICES[0][0],
            attributes="{'data-type', 'link'}",
        )

    def tearDown(self):
        self.file.delete()
        self.page.delete()

    def test_link_instance(self):
        instance = self.link
        instance = Link.objects.all()
        self.assertEqual(instance.count(), 1)
        instance = Link.objects.first()
        self.assertEqual(instance.template, "default")
        self.assertEqual(instance.name, "My Link")
        self.assertEqual(instance.external_link, "http://www.divio.com")
        self.assertEqual(instance.anchor, "some_id")
        self.assertEqual(instance.mailto, "test@email.com")
        self.assertEqual(instance.phone, "+01 234 567 89")
        self.assertEqual(instance.target, "_blank")
        self.assertEqual(instance.attributes, "{'data-type', 'link'}")
        # test strings
        self.assertEqual(str(instance), "My Link")
        # we test these later in get_link
        instance.internal_link = None
        instance.file_link = None
        self.assertEqual(
            instance.get_short_description(),
            "My Link (http://www.divio.com)",
        )
        instance.name = None
        self.assertEqual(str(instance), "1")
        self.assertEqual(instance.get_short_description(), "http://www.divio.com")
        instance.external_link = None
        instance.internal_link = None
        instance.file_link = None
        instance.phone = None
        instance.mailto = None
        instance.anchor = None
        self.assertEqual(instance.get_short_description(), "<link is missing>")

    def test_get_link(self):
        instance = self.link
        with self.assertRaises(ValidationError):
            # should throw an error as too many values are provided
            instance.clean()
        self.assertEqual(instance.get_link(), "//example.com" + self.page.get_absolute_url())
        instance.internal_link = None
        self.assertEqual(instance.get_link(), self.file.url)
        instance.file_link = None
        self.assertEqual(instance.get_link(), "http://www.divio.com")
        instance.external_link = None
        self.assertEqual(instance.get_link(), "tel:+0123456789")
        instance.phone = None
        self.assertEqual(instance.get_link(), "mailto:test@email.com")
        instance.mailto = None
        self.assertEqual(instance.get_link(), "#some_id")
        # by now the configuration is good again
        instance.clean()
        instance.anchor = None
        with self.assertRaises(ValidationError):
            # this should error again as no link is defined
            instance.clean()
        # now we allow the link to be empty
        instance.link_is_optional = True
        instance.clean()

    def test_not_allowed_attributes(self):
        instance = self.link
        instance.internal_link = None
        instance.external_link = None
        instance.phone = None
        instance.file_link = None
        with self.assertRaises(ValidationError):
            # anchor is not compatible with all fields (e.g. phone, mailto)
            instance.clean()
        instance.mailto = None
        instance.clean()
